package com.example.plugin;

import java.util.Base64;

import org.apache.cordova.*;
import org.json.JSONArray;
import org.json.JSONException;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.util.Log;
import wangpos.sdk4.libbasebinder.Printer;
import wangpos.sdk4.libbasebinder.Printer.BarcodeType;
import wangpos.sdk4.libbasebinder.Printer.BarcodeWidth;
import wangpos.sdk4.libbasebinder.Printer.Font;

/**
 * Wrapper class for Wisenet Printer Plugin.
 */
public class Hello extends CordovaPlugin {
	private static final String TAG = "WisenetPrinterPlugin";
	
	private Printer sdkPrinter;

	static Context context = null;

	private CallbackContext callbackContext = null;

	private String lock = "lockAccess";

	@Override
	public void initialize(CordovaInterface cordova, CordovaWebView webView) {
		super.initialize(cordova, webView);
		try {
			if (this.sdkPrinter.printInit() != 0) {
				Log.i("Printer Plugin","Printer Initialization Failed" );
				throw new Exception("Printer Initialization Failed");
			} else {
				Log.i(TAG,"Printer Initialization Successful" );
			}
				
		} catch (Exception e) {
			Log.e(TAG, "Exception occured during initialize", e);
			
		}
	}

	@Override
	public void onDestroy() {
		super.onDestroy();
		try {
			this.sdkPrinter.clearPrintDataCache();
			Log.i(TAG, "Printer data cache cleared");
		} catch (Exception e) {
			Log.e(TAG, "Exception occured during destroy", e);
		}
	}

	/**
	 * Executes the request.
	 *
	 * @param action   The action to execute.
	 * @param args     The exec() arguments in JSON form.
	 * @param callback The callback context used when calling back into JavaScript.
	 *
	 * @return Whether the action was valid.
	 */
	@Override
	public boolean execute(String action, JSONArray args, CallbackContext callbackContext) throws JSONException {
		boolean valid = true;
		this.setCallbackContext(callbackContext);
		if (args.length() != 0) {
			switch (action) {
				case "greet":
						String name = args.getString(0);
						String message = "Hello, " + name;
						callbackContext.success(message);
						break;
//			case "print":
//				showAlertMsg("Action:print", "Print action is invoked");
//				callbackContext.success("prePrint");
//				print(args.getString(0), args, callbackContext);
//				callbackContext.success("Print Successful");
//				break;
			case "printString":
				Log.i(TAG, "PrintString action is invoked");
				showAlertMsg("Action:print", "PrintString action is invoked");
				callbackContext.success("prePrint");
				printString(args.getString(0), Integer.valueOf(args.getString(1)).intValue(), args.getString(2),
						Boolean.valueOf(args.getString(3)), Boolean.valueOf(args.getString(4)));
				Log.i(TAG, "PrintString action invocation is successful");
				callbackContext.success("Print String is Successful");				
				break;
			case "getStatus":
				Log.i(TAG, "getStatus action is invoked");
				showAlertMsg("Action:print", "get Printer Status action is invoked");
				if (getPrinterStatus() == 0) {
					Log.i(TAG, "getStatus action invocation is successful");
					callbackContext.success("Printer Status OK");					
				} else {
					valid = false;
					Log.e(TAG, "getStatus action : No Printer is Connected");
					callbackContext.error("No Printer is Connected");
				}
				break;
			case "printImageBase":
				Log.i(TAG, "printImageBase action is invoked");
				showAlertMsg("Action:print", "Print Image Base action is invoked");
				callbackContext.success("prePrint");
				printImageBase(args.getString(0), callbackContext, Integer.valueOf(args.getString(1)).intValue(),
						Integer.valueOf(args.getString(2)).intValue(), args.getString(3),
						Integer.valueOf(args.getString(4)).intValue());
				Log.i(TAG, "printImageBase action invocation is successful");
				callbackContext.success("Print Image is Successful");
				break;
			case "printPaper":
				Log.i(TAG, "printPaper action is invoked");
				showAlertMsg("Action:print", "Print Paper action is invoked");
				callbackContext.success("prePrint");
				printPaper(Integer.valueOf(args.getString(0)).intValue());
				Log.i(TAG, "printPaper action invocation is successful");
				callbackContext.success("Print Paper is Successful");
				break;
			case "printTextWithFormat":
				Log.i(TAG, "printTextWithFormat action is invoked");
				showAlertMsg("Action:print", "Print Text With Format action is invoked");
				callbackContext.success("prePrint");
				printTextWithFormat(args.getString(0), Integer.valueOf(args.getString(1)).intValue(),
						Float.valueOf(args.getString(2)).floatValue(), Float.valueOf(args.getString(3)).floatValue(),
						args.getString(4), Integer.valueOf(args.getString(5)).intValue(), args.getString(6),
						Boolean.valueOf(args.getString(7)).booleanValue(),
						Boolean.valueOf(args.getString(8)).booleanValue(),
						Boolean.valueOf(args.getString(9)).booleanValue());
				Log.i(TAG, "printTextWithFormat action invocation is successful");
				callbackContext.success("Print Text With Format is Successful");
				break;
			case "print2StringsInALine":
				Log.i(TAG, "print2StringsInALine action is invoked");
				showAlertMsg("Action:print", "Print 2 Strings in a Line action is invoked");
				callbackContext.success("prePrint");
				print2StringsInALine(args.getString(0), args.getString(1),
						Float.valueOf(args.getString(2)).floatValue(), args.getString(3),
						Integer.valueOf(args.getString(4)).intValue(),
						args.getString(5),
						Boolean.valueOf(args.getString(6)).booleanValue(),
						Boolean.valueOf(args.getString(7)).booleanValue(),
						Boolean.valueOf(args.getString(8)).booleanValue());
				Log.i(TAG, "print2StringsInALine action invocation is successful");
				callbackContext.success("Print 2 Strings in a Line is Successful");
				break;
			case "printFinish":
				Log.i(TAG, "printFinish action is invoked");
				showAlertMsg("Action:print", "Print Finish action is invoked");
				callbackContext.success("prePrint");
				printFinish();
				Log.i(TAG, "printFinish action invocation is successful");
				callbackContext.success("Print Finish is Successful");
				break;
			default:
			}
		} else {
			valid = false;
			Log.e(TAG, "execute method : Input action is not provided");
			callbackContext.error("No Inputs (Content, Font, Font Type etc) are Provided");
		}

		return valid;
	}

	/**
	 * Sends the printer status back to the client.
	 *
	 * @param value The argument to pass with.
	 */
	public int getPrinterStatus() {
		int[] value = new int[0];
		int status = 0;
		try {
			status = sdkPrinter.getPrinterStatus(value);
			Log.i(TAG, "getPrinterStatus : " + status);
		} catch (Exception e) {
			Log.e(TAG, "getPrinterStatus error ", e);
		}
		return status;
	}

	/**
	 * Sends the provided content to the printing controller and opens them.
	 *
	 * @param content  The content or file to print.
	 * @param settings Additional settings how to render the content.
	 * @param callback The plugin function to invoke with the result.
	 */
	/*
	private void print(@Nullable String content, JSONArray settings, CallbackContext callback) {
		cordova.getThreadPool().execute(() -> {
			// PrintManager pm = new PrintManager(cordova.getContext());
			// WebView view = (WebView) webView.getView();

			// pm.print(content, settings, view, (boolean completed) ->
			// sendPluginResult(callback, completed));
		});
	}
	*/	

	/**
	 * Prints the Image provided the width, height, alignment and offset.
	 * 
	 * @param bitmap
	 * @param width
	 * @param height
	 * @param align
	 * @param offset
	 */
	public void printImageBase(String completeImageData, CallbackContext callbackContext, int width, int height,
			String align, int offset) {

		if (getPrinterStatus() != 1) {
			Log.e(TAG, "printImageBase : No printer detected");
			callbackContext.error("No printer detected");
			return;
		}

		byte[] decodedByte = Base64.getDecoder().decode(completeImageData);
		Bitmap image = BitmapFactory.decodeByteArray(decodedByte, 0, decodedByte.length);

		synchronized (lock) {
			try {

				this.sdkPrinter.printImageBase(image, width, height, Printer.Align.valueOf(align), offset);
				Log.i(TAG, "printImageBase is successful");
			} catch (Exception e) {
				Log.e(TAG, "printImageBase error", e);
			}
		}

	}

	/**
	 * Prints the Image provided the width, height, alignment and offset.
	 * 
	 * @param bitmap
	 * @param width
	 * @param height
	 * @param align
	 * @param offset
	 */
	public void printImageBase(Bitmap image, int width, int height, String align, int offset) {

		synchronized (lock) {
			try {
				if (image != null) {
					this.sdkPrinter.printImageBase(image, width, height, Printer.Align.valueOf(align), offset);
					Log.i(TAG, "printImageBase is successful");
				} else {
					Log.e(TAG, "printImageBase error : Input Image is Null");
					throw new Exception("Input Image is Null..");
				}

			} catch (Exception e) {
				Log.e(TAG, "printImageBase error", e);
			}
		}

	}

	/**
	 * Prints the String provided the width, height, alignment and offset.
	 * 
	 * @param content
	 * @param fontSize
	 * @param align
	 * @param isBold
	 * @param isItalic
	 */
	public void printString(String content, int fontSize, String align, boolean isBold, boolean isItalic) {
		synchronized (lock) {
			try {
				this.sdkPrinter.printString(content, fontSize, Printer.Align.valueOf(align), isBold, isItalic);
				Log.i(TAG, "printString is successful");
			} catch (Exception e) {
				Log.e(TAG, "printString error", e);
			}
		}
	}

	/**
	 * Prints the barcode for the provided content with the input bar width, height
	 * and offset and for the specific barcode type
	 * 
	 * @param content
	 * @param barcodeType
	 * @param barWidth
	 * @param height
	 * @param offset
	 */
	public void printBarCodeBase(String content, String barcodeType, String barWidth, int height, int offset) {
		try {
			this.sdkPrinter.printBarCodeBase(content, BarcodeType.valueOf(barcodeType), BarcodeWidth.valueOf(barWidth),
					height, offset);
			Log.i(TAG, "printBarCodeBase is successful");
		} catch (Exception e) {
			Log.e(TAG, "printBarCodeBase error", e);
		}
	}

	/**
	 * Prints the QRCode for the input qrcode with the specified width and alignment
	 * type.
	 * 
	 * @param qrCode
	 * @param width
	 * @param type
	 */
	public void printQRCode(String qrCode, int width, String type) {
		try {
			this.sdkPrinter.printQRCode(qrCode, width, Printer.Align.valueOf(type));
			Log.i(TAG, "printQRCode is successful");
		} catch (Exception e) {
			Log.e(TAG, "printQRCode error", e);
		}
	}

	/**
	 * This API is used for Print and roll-out paper after loading print data.
	 * 
	 * @param step
	 */
	public void printPaper(int step) {
		try {
			this.sdkPrinter.printPaper(step);
			Log.i(TAG, "printPaper is successful");
		} catch (Exception e) {
			Log.e(TAG, "printPaper error", e);
		}
	}

	/**
	 * This API is used to print the text with the specified format
	 * 
	 * @param content     The String to be displayed
	 * @param leftOffset  Offset from the left boundary
	 * @param letterWidth Width of the letter space
	 * @param lineSpacing Height of the row
	 * @param font        Font style
	 * @param fontSize
	 * @param align
	 * @param isBold
	 * @param isItalic
	 * @param isUnderLine
	 * @return 0 Only if Success
	 */
	public int printTextWithFormat(String content, int leftOffset, float letterWidth, float lineSpacing, String font,
			int fontSize, String align, boolean isBold, boolean isItalic, boolean isUnderLine) {
		int result = 0;
		try {
			result = this.sdkPrinter.printStringExt(content, leftOffset, letterWidth, lineSpacing, Font.valueOf(font),
					fontSize, Printer.Align.valueOf(align), isBold, isItalic, isUnderLine);
			Log.i(TAG, "printTextWithFormat is successful");
		} catch (Exception e) {
			Log.e(TAG, "printTextWithFormat error", e);
		}
		return result;
	}

	/**
	 * This API is used to print two strings in the same line.
	 * 
	 * @param leftContent  String to be displayed on the left side of the same line
	 * @param rightContent String to be displayed on the right side of the same line
	 * @param lineSpacing  height of the row
	 * @param font         Font style
	 * @param fontSize
	 * @param align
	 * @param isBold       Bold Font Option
	 * @param isItalic     Italic Font Option
	 * @param isUnderLine  UnderLine Font Option
	 * @return 0 Only if Success
	 */
	public int print2StringsInALine(String leftContent, String rightContent, float lineSpacing, String font,
			int fontSize, String align, boolean isBold, boolean isItalic, boolean isUnderLine) {
		int result = 0;
		try {
			result = this.sdkPrinter.print2StringInLine(leftContent, rightContent, lineSpacing, Font.valueOf(font),
					fontSize, Printer.Align.valueOf(align), isBold, isItalic, isUnderLine);
			Log.i(TAG, "print2StringsInALine is successful");
		} catch (Exception e) {
			Log.e(TAG, "print2StringsInALine error", e);
		}
		return result;
	}

	/**
	 * This API is used for Finish the Print routine after print paper
	 * 
	 * @return 0 Only if Success
	 */
	public int printFinish() {
		int result = 0;
		try {
			result = this.sdkPrinter.printFinish();
			Log.i(TAG, "printFinish is successful");
		} catch (Exception e) {
			Log.e(TAG, "printFinish error", e);
		}
		return result;
	}

	private void showAlertMsg(String title, String msg) {
		if (callbackContext != null) {

			PluginResult result = new PluginResult(PluginResult.Status.OK, msg);
			callbackContext.sendPluginResult(result);

		}

	}

	/**
	 * To get the Printer Details
	 * 
	 * @return <code>wangpos.sdk4.libbasebinder.Printer</code>
	 */
	public Printer getSdkPrinter() {
		return sdkPrinter;
	}

	public void setSdkPrinter(Printer sdkPrinter) {
		this.sdkPrinter = sdkPrinter;
	}

	public CallbackContext getCallbackContext() {
		return callbackContext;
	}

	public void setCallbackContext(CallbackContext callbackContext) {
		this.callbackContext = callbackContext;
	}

}
